import win32.windows;
import win32.windef;
import win32.commctrl;
import win32frame.Frame;
import win32frame.WindowMenu;
import win32frame.StatusBar;
import win32frame.MainWindow ;
import win32frame.MsgBox ;

import std.stdio;
import std.string;
import std.utf;

import Sound ;

const SRATE    = 11025 ;
const PLAYTIME =    60 ;

enum MENUCODE{
  FILE_OPEN    = 0x101,
  FILE_EXIT    = 0x104,
  HELP_VERSION = 0x111
} ;

class MainFrame : MainWindow {
  WindowMenu   menuroot ;
  StatusBar    statusbar ;
  MsgBox       msgbox ;
 
  HFONT        hFont;
  HWAVEOUT     hWave;
  WAVEFORMATEX wfe;
  WAVEHDR      whdr;
  char[]       bWave;
  

  this(string name, string ClassName) {
    winc.style          = 0;
    winc.cbClsExtra     = 0;
    winc.cbWndExtra     = 0;
    winc.hIcon          = LoadIcon(null , cast(wchar*)(IDI_APPLICATION));
    winc.hCursor        = LoadCursor(null , cast(wchar*)(IDC_ARROW));
    winc.hbrBackground  = CreateSolidBrush(GetSysColor(COLOR_BTNFACE)) ;
    winc.lpszMenuName   = null;
    winc.lpszClassName  = toUTFz!(wchar*)(ClassName) ;
    super(name,ClassName) ;
    
  }

  LRESULT proc(HWND hwnd , UINT msg , WPARAM wp , LPARAM lp){
    switch (msg) {
    case WM_DESTROY:
      waveOutClose(hWave);
      DeleteObject(hFont);
      PostQuitMessage(0);
      break ;
      
    case WM_CREATE:
      menuroot = new WindowMenu() ;
      menuroot.create() ;
      menuroot.addMenu(         MFT_STRING, "File"      , 0x000 ) ;
      menuroot.addItem( "File", MFT_STRING, "MML Open"  , MENUCODE.FILE_OPEN ) ;
      menuroot.addItem( "File", MFT_STRING, "Exit"      , MENUCODE.FILE_EXIT ) ;
      menuroot.addMenu(         MFT_STRING, "Help"      , 0x001 ) ;
      menuroot.addItem( "Help", MFT_STRING, "Version"   , MENUCODE.HELP_VERSION ) ;
      SetMenu(hwnd , menuroot.getHMENU());
      
      statusbar = new StatusBar() ;
      statusbar.addStyle(CCS_BOTTOM | SBARS_SIZEGRIP) ;
      addStatusBar(statusbar, "") ;
      
      hFont = CreateFont(
			 14 , 0 , 0 , 0 , FW_REGULAR , FALSE , FALSE , FALSE ,
			 SHIFTJIS_CHARSET , OUT_DEFAULT_PRECIS ,
			 CLIP_DEFAULT_PRECIS , DEFAULT_QUALITY , 
			 VARIABLE_PITCH, null
			 );

      bWave.length = SRATE*PLAYTIME;
      wfe.wFormatTag      = WAVE_FORMAT_PCM ;
      wfe.nChannels       = 1 ;
      wfe.nSamplesPerSec  = SRATE ;
      wfe.nAvgBytesPerSec = SRATE ;
      wfe.wBitsPerSample  = 8 ;
      wfe.nBlockAlign     = cast(ushort)(wfe.nChannels * wfe.wBitsPerSample /8) ;
      //      waveOutOpen(&hWave , WAVE_MAPPER , &wfe , cast(uint)(hwnd), 0 , CALLBACK_WINDOW);
      
      whdr.lpData         = bWave.ptr;
      whdr.dwBufferLength = SRATE*PLAYTIME;
      whdr.dwFlags        = WHDR_BEGINLOOP | WHDR_ENDLOOP;
      whdr.dwLoops        = 1;
      //      waveOutPrepareHeader(hWave , &whdr , WAVEHDR.sizeof);
      break ;
      
    case MM_WOM_OPEN:
      //    case MM_WOM_DONE:
      waveOutWrite(cast(HWAVEOUT)(wp) , &whdr , WAVEHDR.sizeof);
      break ;
    case MM_WOM_CLOSE:
      //      waveOutUnprepareHeader(cast(HWAVEOUT)(wp) , &whdr , WAVEHDR.sizeof);
      break ;
		
    case WM_MOVE:
      break ;
      
    case WM_SIZE:
      break;
      
    case WM_LBUTTONDOWN:
    case WM_LBUTTONUP:
    case WM_LBUTTONDBLCLK:
    case WM_RBUTTONDOWN:
    case WM_RBUTTONUP:
    case WM_RBUTTONDBLCLK:
    case WM_MOUSEMOVE:
      break ;

    case WM_COMMAND:
      switch( LOWORD(wp) ){
      case MENUCODE.FILE_OPEN : //MML Open
	string fname ;
	fname = getLoadFileName("MML {*.mml}\0*.mml\0ALL {*.*}\0*.*\0") ;
	if( fname.length != 0 ){
	  waveOutReset(hWave) ;
	  waveOutClose(hWave);
	  Sound snd = new Sound() ;
	  snd.readMML(fname) ;
	  snd.rendWave(bWave,SRATE) ;
	  waveOutOpen(&hWave , WAVE_MAPPER , &wfe , cast(DWORD_PTR)(hwnd), 0 , CALLBACK_WINDOW);
	  waveOutPrepareHeader(hWave , &whdr , WAVEHDR.sizeof);
	}
	break ;
      case MENUCODE.FILE_EXIT : //Exit
	SendMessage( hwnd, WM_CLOSE , 0,0 ) ;
	break ;
      case MENUCODE.HELP_VERSION :
        msgbox = new MsgBox(hwnd, format("PSGtest Ver0.01"), " ",MB_OK) ;
        msgbox.create() ;
        break ;
	
      default :	break ;
      }
      break;
      
    default :
      return( DefWindowProc(hwnd , msg , wp , lp) );
    }
    return(0);
  }

  string getLoadFileName(string filematch){
    OPENFILENAME ofn ;
    wchar[]      wfname ;
    string       retstr ;

    wfname.length=MAX_PATH ;
    wfname[0]=0 ;
    
    ofn.lStructSize = OPENFILENAME.sizeof ;
    ofn.hwndOwner   = getHWND() ;
    if( filematch==null ){
      ofn.lpstrFilter = null ;
    }else{
      ofn.lpstrFilter = toUTFz!(wchar*)(filematch) ;
    }    
    ofn.lpstrCustomFilter = null ;
    ofn.lpstrFile   = cast(wchar*)(wfname.ptr) ;
    ofn.nMaxFile    = MAX_PATH ;
    ofn.Flags       = OFN_FILEMUSTEXIST ;

    GetOpenFileName(&ofn) ;

    int len ;
    for( len=0 ; len<wfname.length ; len++ ){
      if( wfname[len]==0 ) break ;
    }
    wfname.length=len ;
    
    retstr = std.utf.toUTF8(wfname.idup) ;
    return(retstr) ;
  }
}
